@@ -207,10 +207,11 @@ SQLRETURN SQLBrowseConnect(
     char *driver, *dsn;
     char lib_name[ INI_MAX_PROPERTY_VALUE + 1 ];
     char driver_name[ INI_MAX_PROPERTY_VALUE + 1 ];
-    char in_str[ BUFFER_LEN ];
+    char in_str_buf[ BUFFER_LEN ];
+    char *in_str;
+    SQLSMALLINT in_str_len;
     SQLRETURN ret;
     SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ], s2[ 100 + LOG_MESSAGE_LEN];
-	SQLWCHAR *uc_in_str;
     int warnings = 0;
 
     /*
@@ -269,7 +270,7 @@ SQLRETURN SQLBrowseConnect(
                 ERROR_08002, NULL, 
                 connection -> environment -> requested_version );
 
-        return function_return( IGNORE_THREAD, connection, SQL_ERROR );
+        return function_return_nodrv( IGNORE_THREAD, connection, SQL_ERROR );
     }
 
     thread_protect( SQL_HANDLE_DBC, connection );
@@ -286,7 +287,7 @@ SQLRETURN SQLBrowseConnect(
                 ERROR_HY090, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -330,10 +331,9 @@ SQLRETURN SQLBrowseConnect(
 
                 __release_conn( &con_struct );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
-            __generate_connection_string( &con_struct, in_str, sizeof( in_str ));
             strcpy( connection -> dsn, "" );
         }
         else
@@ -357,11 +357,9 @@ SQLRETURN SQLBrowseConnect(
                         ERROR_IM012, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
-            __generate_connection_string( &con_struct, in_str, sizeof( in_str ));
-
             /*
              * look up the dsn in the ini file
              */
@@ -379,12 +377,13 @@ SQLRETURN SQLBrowseConnect(
                         connection -> environment -> requested_version );
                 __release_conn( &con_struct );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
             strcpy( connection -> dsn, dsn );
         }
 
+        __generate_connection_string( &con_struct, in_str_buf, sizeof( in_str_buf ));
         __release_conn( &con_struct );
 
         /*
@@ -401,10 +400,11 @@ SQLRETURN SQLBrowseConnect(
                     LOG_INFO,
                     "Error: connect_part_one fails" );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
-        if ( !CHECK_SQLBROWSECONNECT( connection ))
+        if ( !CHECK_SQLBROWSECONNECTW( connection ) &&
+            !CHECK_SQLBROWSECONNECT( connection ))
         {
             dm_log_write( __FILE__, 
                         __LINE__, 
@@ -418,20 +418,15 @@ SQLRETURN SQLBrowseConnect(
                     ERROR_IM001, NULL,
                     connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
+        in_str = in_str_buf;
+        in_str_len = strlen(in_str);
     }
     else
     {
-        if ( len_conn_str_in == SQL_NTS )
-        {
-            strcpy( in_str, (char*)conn_str_in );
-        }
-        else
-        {
-            memcpy( in_str, conn_str_in, len_conn_str_in );
-            in_str[ len_conn_str_in ] = '\0';
-        }
+        in_str = (char*)conn_str_in;
+        in_str_len = len_conn_str_in == SQL_NTS ? strlen(in_str) : len_conn_str_in;
     }
 
     if (CHECK_SQLBROWSECONNECT( connection ))
@@ -439,7 +434,7 @@ SQLRETURN SQLBrowseConnect(
         ret = SQLBROWSECONNECT( connection,
             connection -> driver_dbc,
             in_str,
-            strlen( in_str ),
+            in_str_len,
             conn_str_out,
             conn_str_out_max,
             ptr_conn_str_out );
@@ -449,22 +444,29 @@ SQLRETURN SQLBrowseConnect(
     else if (CHECK_SQLBROWSECONNECTW( connection ))
     {
         int wlen;
+	    SQLWCHAR *uc_in_str = ansi_to_unicode_alloc((SQLCHAR*)in_str,SQL_NTS,connection, &wlen);
+        SQLWCHAR *uc_out_str = conn_str_out ? malloc( (conn_str_out_max + 1) * sizeof(SQLWCHAR) ) : 0;
 
-        uc_in_str = ansi_to_unicode_alloc((SQLCHAR*)in_str,SQL_NTS,connection, &wlen);
-        
         ret = SQLBROWSECONNECTW( connection,
             connection -> driver_dbc,
             uc_in_str,
             wlen,
-            conn_str_out,
+            uc_out_str,
             conn_str_out_max,
             ptr_conn_str_out );
         
         if(uc_in_str)
-             free(uc_in_str);
+            free(uc_in_str);
 
-        connection->unicode_driver = 1;
+        if(uc_out_str)
+        {
+            unicode_to_ansi_copy((char*) conn_str_out, conn_str_out_max, uc_out_str, SQL_NTS, connection, NULL );
+            if (*ptr_conn_str_out < conn_str_out_max)
+                *ptr_conn_str_out = strlen((char*)conn_str_out);
+            free(uc_out_str);
+        }
         
+        connection->unicode_driver = 1;
     }
     else
     {
@@ -479,90 +481,124 @@ SQLRETURN SQLBrowseConnect(
         __post_internal_error( &connection -> error,
                                ERROR_IM001, NULL,
                                connection -> environment -> requested_version );
-          return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     } 
 
     if ( !SQL_SUCCEEDED( ret ) || ret == SQL_NEED_DATA )
     {
-        SQLCHAR sqlstate[ 6 ];
-        SQLINTEGER native_error;
-        SQLSMALLINT ind;
-        SQLCHAR message_text[ SQL_MAX_MESSAGE_LENGTH + 1 ];
-        SQLRETURN eret;
-
         /*
          * get the error from the driver before
-         * looseing the connection
+         * losing the connection
          */
-        if ( CHECK_SQLERROR( connection ))
+        if ( connection -> unicode_driver )
         {
-            do
+            if ( CHECK_SQLGETDIAGFIELDW( connection ) &&
+                    CHECK_SQLGETDIAGRECW( connection ))
             {
-                eret = SQLERROR( connection,
-                    SQL_NULL_HENV,
+                extract_diag_error_w( SQL_HANDLE_DBC,
                         connection -> driver_dbc,
-                        SQL_NULL_HSTMT,
-                        sqlstate,
-                        &native_error,
-                        message_text,
-                        sizeof( message_text ),
-                        &ind );
-
-
-                if ( SQL_SUCCEEDED( eret ))
-                {
-                    __post_internal_error_ex( &connection -> error,
-                            sqlstate,
-                            native_error,
-                            message_text,
-                            SUBCLASS_ODBC, SUBCLASS_ODBC );
-
-                    sprintf( connection -> msg, "\t\tDIAG [%s] %s",
-                            sqlstate, message_text );
-
-                    dm_log_write_diag( connection -> msg );
-                }
+                        connection,
+                        &connection -> error,
+                        ret,
+                        1 );
+            }
+            else if ( CHECK_SQLERRORW( connection )) 
+            {
+                extract_sql_error_w( SQL_NULL_HENV, 
+                        connection -> driver_dbc, 
+                        SQL_NULL_HSTMT, 
+                        connection,
+                        &connection -> error, 
+                        ret );
+            }
+            else if ( CHECK_SQLGETDIAGFIELD( connection ) &&
+                    CHECK_SQLGETDIAGREC( connection ))
+            {
+                extract_diag_error( SQL_HANDLE_DBC,
+                        connection -> driver_dbc,
+                        connection,
+                        &connection -> error,
+                        ret,
+                        1 );
+            }
+            else if ( CHECK_SQLERROR( connection )) 
+            {
+                extract_sql_error( SQL_NULL_HENV, 
+                        connection -> driver_dbc, 
+                        SQL_NULL_HSTMT, 
+                        connection,
+                        &connection -> error, 
+                        ret );
+            }
+            else 
+            {
+                __post_internal_error( &connection -> error,
+                    ERROR_HY000, "Driver returned SQL_ERROR or SQL_SUCCESS_WITH_INFO but no error reporting API found",
+                    connection -> environment -> requested_version );
             }
-            while( SQL_SUCCEEDED( eret ));
         }
-        else if ( CHECK_SQLGETDIAGREC( connection ))
+        else
         {
-            int rec = 1;
-
-            do
+            if ( CHECK_SQLGETDIAGFIELD( connection ) &&
+                    CHECK_SQLGETDIAGREC( connection ))
             {
-                eret = SQLGETDIAGREC( connection,
-                        SQL_HANDLE_DBC,
+                extract_diag_error( SQL_HANDLE_DBC,
                         connection -> driver_dbc,
-                        rec ++,
-                        sqlstate,
-                        &native_error,
-                        message_text,
-                        sizeof( message_text ),
-                        &ind );
-
-                if ( SQL_SUCCEEDED( eret ))
-                {
-                    __post_internal_error_ex( &connection -> error,
-                            sqlstate,
-                            native_error,
-                            message_text,
-                            SUBCLASS_ODBC, SUBCLASS_ODBC );
-
-                    sprintf( connection -> msg, "\t\tDIAG [%s] %s",
-                            sqlstate, message_text );
-
-                    dm_log_write_diag( connection -> msg );
-                }
+                        connection,
+                        &connection -> error,
+                        ret,
+                        1 );
+            }
+            else if ( CHECK_SQLERROR( connection )) 
+            {
+                extract_sql_error( SQL_NULL_HENV, 
+                        connection -> driver_dbc, 
+                        SQL_NULL_HSTMT, 
+                        connection,
+                        &connection -> error, 
+                        ret );
+            }
+            else if ( CHECK_SQLGETDIAGFIELDW( connection ) &&
+                    CHECK_SQLGETDIAGRECW( connection ))
+            {
+                extract_diag_error_w( SQL_HANDLE_DBC,
+                        connection -> driver_dbc,
+                        connection,
+                        &connection -> error,
+                        ret,
+                        1 );
+            }
+            else if ( CHECK_SQLERRORW( connection )) 
+            {
+                extract_sql_error_w( SQL_NULL_HENV, 
+                        connection -> driver_dbc, 
+                        SQL_NULL_HSTMT, 
+                        connection,
+                        &connection -> error, 
+                        ret );
+            }
+            else 
+            {
+                __post_internal_error( &connection -> error,
+                    ERROR_HY000, "Driver returned SQL_ERROR or SQL_SUCCESS_WITH_INFO but no error reporting API found",
+                    connection -> environment -> requested_version );
             }
-            while( SQL_SUCCEEDED( eret ));
         }
 
     	if ( ret != SQL_NEED_DATA ) 
 		{
-        	__disconnect_part_one( connection );
-            __disconnect_part_four( connection );       /* release unicode handles */
-        	connection -> state = STATE_C2;
+            /* If an error occurred during SQLBrowseConnect, we need to keep the
+               connection in the same state (C2 or C3). This allows the application
+               to either try the SQLBrowseConnect again, or disconnect an active
+               browse session with SQLDisconnect. Otherwise the driver may continue
+               to have an active connection while the DM thinks it does not,
+               causing more errors later on. */
+            if ( connection -> state == STATE_C2 )
+            {
+                /* only disconnect and unload if we never started browsing */
+                __disconnect_part_one( connection );
+                __disconnect_part_four( connection );  /* release unicode handles - also sets state to C2 */
+            }
 		}
 		else 
 		{
@@ -579,7 +615,7 @@ SQLRETURN SQLBrowseConnect(
 
         if( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( IGNORE_THREAD, connection, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, connection, ret, TRUE, DEFER_R0 );
         }
 
         if ( !__connect_part_two( connection ))
@@ -601,7 +637,7 @@ SQLRETURN SQLBrowseConnect(
                         connection -> msg );
             }
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
     }
 
@@ -620,12 +656,10 @@ SQLRETURN SQLBrowseConnect(
                 connection -> msg );
     }
 
-    connection -> unicode_driver = 0;
-
     if ( warnings && ret == SQL_SUCCESS )
     {
         ret = SQL_SUCCESS_WITH_INFO;
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret );
+    return function_return_nodrv( SQL_HANDLE_DBC, connection, ret );
 }
