@@ -223,7 +223,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                 ERROR_HY090, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -241,7 +241,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( name_length1 && server_name )
@@ -262,7 +262,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                         ERROR_HY090, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
         }
         else
@@ -281,7 +281,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                         ERROR_HY090, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
         }
         memcpy( dsn, server_name, sizeof( dsn[ 0 ] ) * len );
@@ -299,7 +299,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                 ERROR_IM010, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
     else
     {
@@ -317,7 +317,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
 
     unicode_to_ansi_copy((char*) ansi_dsn, sizeof( ansi_dsn ), dsn, sizeof( ansi_dsn ), NULL, NULL );
 
-    if ( !__find_lib_name((char*) ansi_dsn, lib_name, driver_name ))
+    if ( !*ansi_dsn || !__find_lib_name((char*) ansi_dsn, lib_name, driver_name ))
     {
         /*
          * if not found look for a default
@@ -335,7 +335,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                     ERROR_IM002, NULL,
                     connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -355,7 +355,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
     {
         __disconnect_part_four( connection );       /* release unicode handles */
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( !CHECK_SQLCONNECTW( connection ) &&
@@ -373,16 +373,14 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                 ERROR_IM001, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( CHECK_SQLCONNECTW( connection ))
     {
         if ( CHECK_SQLSETCONNECTATTR( connection ))
         {
-            int lret;
-                
-            lret = SQLSETCONNECTATTR( connection,
+            SQLSETCONNECTATTR( connection,
                     connection -> driver_dbc,
                     SQL_ATTR_ANSI_APP,
                     SQL_AA_FALSE,
@@ -463,7 +461,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                             sqlstate,
                             &native_error,
                             message_text,
-                            sizeof( message_text ),
+                            sizeof( message_text )/sizeof(SQLWCHAR),
                             &ind );
 
 
@@ -501,7 +499,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
                             sqlstate,
                             &native_error,
                             message_text,
-                            sizeof( message_text ),
+                            sizeof( message_text )/sizeof(SQLWCHAR),
                             &ind );
 
 
@@ -621,7 +619,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
             __disconnect_part_one( connection );
             __disconnect_part_four( connection );       /* release unicode handles */
 
-            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
         }
     }
 
@@ -658,7 +656,7 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
 
         connection -> state = STATE_C3;
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( log_info.log_flag ) 
@@ -679,5 +677,5 @@ SQLRETURN SQLConnectW( SQLHDBC connection_handle,
         ret_from_connect = SQL_SUCCESS_WITH_INFO;
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+    return function_return_nodrv( SQL_HANDLE_DBC, connection, ret_from_connect );
 }
