@@ -333,10 +333,28 @@ char *tmp;
     cp = con_str -> list;
     while( cp )
     {
-        tmp = malloc( strlen( cp -> keyword ) + strlen( cp -> attribute ) + 10 );
-        if( strcasecmp( cp -> keyword, "DRIVER" ) == 0 )
+        size_t attrlen = strlen( cp -> attribute );
+        int use_esc = isspace( *(cp -> attribute ) ) || ( attrlen && isspace( cp->attribute[attrlen - 1] ));
+        for ( tmp = cp -> attribute; *tmp; tmp++ )
         {
-            sprintf( tmp, "%s={%s};", cp -> keyword, cp -> attribute );
+            use_esc |= (*tmp == '{') || (*tmp == '}');
+            attrlen += (*tmp == '}');
+        }
+        tmp = malloc( strlen( cp -> keyword ) + attrlen + 10 );
+        if( use_esc )
+        {
+            char *tmp2 = tmp + sprintf( tmp, "%s={", cp -> keyword );
+            char *tmp3;
+            for( tmp3 = cp -> attribute; *tmp3 ; tmp3++ )
+            {
+                *tmp2++ = *tmp3;
+                if ( '}' == *tmp3++ )
+                {
+                    *tmp2++ = '}';
+                }
+            }
+            *tmp2++ = '}';
+            *tmp2++ = 0;
         }
         else
         {
@@ -364,17 +382,22 @@ int len;
 
     *keyword = *value = NULL;
 
-    ptr = *cp;
+    while ( isspace( **cp ) || **cp == ';' )
+    {
+        (*cp)++;
+    }
 
     if ( !**cp )
         return;
 
+    ptr = *cp;
+
     /* 
      * To handle the case attribute in which the attribute is of the form
      * "ATTR;" instead of "ATTR=VALUE;"
      */
 
-    while ( **cp && **cp != ';' && **cp != '=' )
+    while ( **cp && **cp != '=' )
     {
         (*cp)++;
     }
@@ -387,51 +410,48 @@ int len;
     memcpy( *keyword, ptr, len );
     (*keyword)[ len ] = '\0';
 
-    if (**cp != ';') {
-		(*cp)++;
-	}
+    (*cp)++;
 
-    ptr = *cp;
-
-    if ( strcasecmp( *keyword, "DRIVER" ) == 0 )
+    if ( **cp == '{' )
     {
-        if ( **cp && **cp == '{' )
+        /* escaped with '{' - all characters until next '}' not followed by '}', or
+           end of string, are part of the value */
+        int i = 0;
+        ptr = ++*cp;
+        while ( **cp && (**cp != '}' || (*cp)[1] == '}') )
         {
-            (*cp)++;
-            ptr ++;
-            while ( **cp && **cp != '}' )
+            if ( **cp == '}' )
                 (*cp)++;
-
-            len = *cp - ptr;
-            *value = malloc( len + 1 );
-            memcpy( *value, ptr , len );
-            (*value)[ len ] = '\0';
             (*cp)++;
         }
-        else
+        len = *cp - ptr;
+        *value = malloc( len + 1 );
+        while( ptr < *cp )
         {
-            while ( **cp && **cp != ';' )
-                (*cp)++;
-
-            len = *cp - ptr;
-            *value = malloc( len + 1 );
-            memcpy( *value, ptr, len );
-            (*value)[ len ] = '\0';
+            if ( ((*value)[i++] = *ptr++) == '}')
+            {
+                ptr++;
+            }
+        }
+        (*value)[i] = 0;
+        if ( **cp == '}' )
+        {
+            (*cp)++;
         }
     }
     else
     {
+        /* non-escaped: all characters until ';' or end of string are value */
+        ptr = *cp;
         while ( **cp && **cp != ';' )
+        {
             (*cp)++;
-
+        }
         len = *cp - ptr;
         *value = malloc( len + 1 );
         memcpy( *value, ptr, len );
-        (*value)[ len ] = '\0';
+        (*value)[ len ] = 0;
     }
-
-    if ( **cp )
-        (*cp)++;
 }
 
 struct con_pair * __get_pair( char ** cp )
@@ -745,7 +765,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_HY090, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( driver_completion == SQL_DRIVER_PROMPT &&
@@ -761,7 +781,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_HY092, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( driver_completion != SQL_DRIVER_PROMPT &&
@@ -779,7 +799,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_HY110, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     /*
@@ -798,7 +818,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     /*
@@ -831,7 +851,7 @@ SQLRETURN SQLDriverConnect(
 			!__get_attribute_value( &con_struct, "FILEDSN" ))
 		{
 			int ret;
-			SQLCHAR returned_dsn[ 128 ], *prefix, *target;
+			SQLCHAR returned_dsn[ 1025 ], *prefix, *target;
 
 			/*
 			 * try and call GUI to obtain a DSN
@@ -934,7 +954,7 @@ SQLRETURN SQLDriverConnect(
 
         __release_conn( &con_struct );
 
-        return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+        return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
     }
 
     /*
@@ -1047,7 +1067,7 @@ SQLRETURN SQLDriverConnect(
                                     free( save_filedsn );
                                 }
                         
-                                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
                             }
 
 							if ( strlen((char*) conn_str_in ) > 0 )
@@ -1103,7 +1123,7 @@ SQLRETURN SQLDriverConnect(
                                     free( save_filedsn );
                                 }
                         
-                                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
                             }
 
 							if ( strlen((char*) conn_str_in ) > 0 )
@@ -1123,6 +1143,7 @@ SQLRETURN SQLDriverConnect(
                                         ERROR_01004, NULL,
                                         connection -> environment -> requested_version );
                             }
+                            free( str1 );
                         }
                         cp = cp -> next;
                     }
@@ -1142,6 +1163,10 @@ SQLRETURN SQLDriverConnect(
             __parse_connection_string( &con_struct,
                     (char*)conn_str_in, len_conn_str_in );
         }
+        else
+        {
+            save_filedsn = NULL;
+        }
     }
     else
     {
@@ -1175,7 +1200,7 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
                         
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         strcpy( driver_name, driver );
@@ -1226,7 +1251,7 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         if ( strlen( dsn ) > SQL_MAX_DSN_LENGTH )
@@ -1247,7 +1272,7 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         /*
@@ -1272,7 +1297,7 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         strcpy( connection -> dsn, dsn );
@@ -1308,7 +1333,7 @@ SQLRETURN SQLDriverConnect(
 
         __disconnect_part_four( connection );       /* release unicode handles */
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( !CHECK_SQLDRIVERCONNECT( connection ) &&
@@ -1330,7 +1355,7 @@ SQLRETURN SQLDriverConnect(
             free( save_filedsn );
         }
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( CHECK_SQLDRIVERCONNECT( connection ))
@@ -1387,7 +1412,7 @@ SQLRETURN SQLDriverConnect(
 
                     if ( SQL_SUCCEEDED( ret ))
                     {
-                        __post_internal_error_ex( &connection -> error,
+                        __post_internal_error_ex_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -1420,7 +1445,7 @@ SQLRETURN SQLDriverConnect(
 
                     if ( SQL_SUCCEEDED( ret ))
                     {
-                        __post_internal_error_ex( &connection -> error,
+                        __post_internal_error_ex_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -1460,7 +1485,7 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
-            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
         }
 		connection -> unicode_driver = 0;
     }
@@ -1475,9 +1500,7 @@ SQLRETURN SQLDriverConnect(
 
         if ( CHECK_SQLSETCONNECTATTR( connection ))
         {
-            int lret;
-                
-            lret = SQLSETCONNECTATTR( connection,
+            SQLSETCONNECTATTR( connection,
                     connection -> driver_dbc,
                     SQL_ATTR_ANSI_APP,
                     SQL_AA_FALSE,
@@ -1534,7 +1557,7 @@ SQLRETURN SQLDriverConnect(
                     {
                         SQLCHAR *as1, *as2;
 
-                        __post_internal_error_ex_w( &connection -> error,
+                        __post_internal_error_ex_w_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -1575,7 +1598,7 @@ SQLRETURN SQLDriverConnect(
                     {
                         SQLCHAR *as1, *as2;
 
-                        __post_internal_error_ex_w( &connection -> error,
+                        __post_internal_error_ex_w_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -1620,7 +1643,12 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
-            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+            if ( s1 )
+            {
+                free( s1 );
+            }
+
+            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
         }
         else
         {
@@ -1670,7 +1698,7 @@ SQLRETURN SQLDriverConnect(
             free( save_filedsn );
         }
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( log_info.log_flag )
@@ -1812,5 +1840,5 @@ SQLRETURN SQLDriverConnect(
         ret_from_connect = SQL_SUCCESS_WITH_INFO;
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+    return function_return_nodrv( SQL_HANDLE_DBC, connection, ret_from_connect );
 }
