@@ -152,7 +152,7 @@ int got_driver = 0;    /* if we have a DRIVER or FILEDSN then ignore any DSN */
         local_str = malloc( len + 1 );
     }
 
-    unicode_to_ansi_copy( local_str, len, str, len, NULL, NULL );
+    unicode_to_ansi_copy( local_str, len+1, str, len, NULL, NULL );
 
     if ( !local_str || strlen( local_str ) == 0 ||
         ( strlen( local_str ) == 1 && *local_str == ';' ))
@@ -307,7 +307,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_HY090, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( driver_completion == SQL_DRIVER_PROMPT &&
@@ -323,7 +323,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_HY092, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( driver_completion != SQL_DRIVER_PROMPT &&
@@ -341,7 +341,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_HY110, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     /*
@@ -360,7 +360,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     /*
@@ -409,19 +409,21 @@ SQLRETURN SQLDriverConnectW(
 			!__get_attribute_value( &con_struct, "FILEDSN" ))
 		{
 			int ret;
-			SQLCHAR returned_dsn[ 128 ], *prefix, *target;
+			SQLWCHAR returned_wdsn[ 1025 ];
+			SQLCHAR *prefix, *target, returned_dsn[ 1025 ];
 
 			/*
 			 * try and call GUI to obtain a DSN
 			 */
 
-			ret = _SQLDriverConnectPrompt( hwnd, returned_dsn, sizeof( returned_dsn ));
-			if ( !ret || returned_dsn[ 0 ] == '\0' ) 
+			ret = _SQLDriverConnectPromptW( hwnd, returned_wdsn, sizeof( returned_wdsn ));
+			if ( !ret || returned_wdsn[ 0 ] == 0 ) 
 			{
         		__append_pair( &con_struct, "DSN", "DEFAULT" );
 			}
 			else 
 			{
+                unicode_to_ansi_copy((char*) returned_dsn, sizeof( returned_dsn ), returned_wdsn, SQL_NTS, connection, NULL );
 				prefix = returned_dsn;
 				target = (SQLCHAR*)strchr( (char*)returned_dsn, '=' );
 				if ( target ) 
@@ -504,7 +506,7 @@ SQLRETURN SQLDriverConnectW(
                     connection -> environment -> requested_version );
             __release_conn( &con_struct );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         if ( strlen( dsn ) > SQL_MAX_DSN_LENGTH )
@@ -519,7 +521,7 @@ SQLRETURN SQLDriverConnectW(
                     ERROR_IM012, NULL,
                     connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         /*
@@ -539,7 +541,7 @@ SQLRETURN SQLDriverConnectW(
                     connection -> environment -> requested_version );
             __release_conn( &con_struct );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
         strcpy( connection -> dsn, dsn );
@@ -569,7 +571,7 @@ SQLRETURN SQLDriverConnectW(
     if ( !__connect_part_one( connection, lib_name, driver_name, &warnings ))
     {
         __disconnect_part_four( connection );       /* release unicode handles */
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( !CHECK_SQLDRIVERCONNECTW( connection ) &&
@@ -587,16 +589,14 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_IM001, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( CHECK_SQLDRIVERCONNECTW( connection ))
     {
         if ( CHECK_SQLSETCONNECTATTR( connection ))
         {
-            int lret;
-                
-            lret = SQLSETCONNECTATTR( connection,
+            SQLSETCONNECTATTR( connection,
                     connection -> driver_dbc,
                     SQL_ATTR_ANSI_APP,
                     SQL_AA_FALSE,
@@ -643,7 +643,7 @@ SQLRETURN SQLDriverConnectW(
 
                     if ( SQL_SUCCEEDED( ret ))
                     {
-                        __post_internal_error_ex_w( &connection -> error,
+                        __post_internal_error_ex_w_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -671,7 +671,7 @@ SQLRETURN SQLDriverConnectW(
 
                     if ( SQL_SUCCEEDED( ret ))
                     {
-                        __post_internal_error_ex_w( &connection -> error,
+                        __post_internal_error_ex_w_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -701,7 +701,7 @@ SQLRETURN SQLDriverConnectW(
                         LOG_INFO,
                         connection -> msg );
 
-                return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+                return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
             }
         }
         connection -> unicode_driver = 1;
@@ -709,20 +709,21 @@ SQLRETURN SQLDriverConnectW(
     else
     {
         char *in_str, *out_str;
-        int len;
+        int in_len, len;
 
         if ( conn_str_in )
         {
             if ( len_conn_str_in == SQL_NTS )
             {
-                len = wide_strlen( conn_str_in ) + sizeof( SQLWCHAR );
+                len = wide_strlen( conn_str_in );
             }
             else
             {
-                len = len_conn_str_in + sizeof( SQLWCHAR );
+                len = len_conn_str_in;
             }
-            in_str = malloc( len );
-            unicode_to_ansi_copy( in_str, len, conn_str_in, len, connection, NULL );
+            in_len = len + 1;
+            in_str = malloc( in_len );
+            unicode_to_ansi_copy( in_str, in_len, conn_str_in, len, connection, NULL );
         }
         else
         {
@@ -792,7 +793,7 @@ SQLRETURN SQLDriverConnectW(
 
                     if ( SQL_SUCCEEDED( ret ))
                     {
-                        __post_internal_error_ex( &connection -> error,
+                        __post_internal_error_ex_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -819,7 +820,7 @@ SQLRETURN SQLDriverConnectW(
 
                     if ( SQL_SUCCEEDED( ret ))
                     {
-                        __post_internal_error_ex( &connection -> error,
+                        __post_internal_error_ex_noprefix( &connection -> error,
                                 sqlstate,
                                 native_error,
                                 message_text,
@@ -848,7 +849,7 @@ SQLRETURN SQLDriverConnectW(
                         LOG_INFO,
                         connection -> msg );
 
-                return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+                return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
             }
         }
         connection -> unicode_driver = 0;
@@ -882,7 +883,7 @@ SQLRETURN SQLDriverConnectW(
         __disconnect_part_one( connection );
         __disconnect_part_four( connection );       /* release unicode handles */
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
     if ( log_info.log_flag )
@@ -920,5 +921,5 @@ SQLRETURN SQLDriverConnectW(
         ret_from_connect = SQL_SUCCESS_WITH_INFO;
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
+    return function_return_nodrv( SQL_HANDLE_DBC, connection, ret_from_connect );
 }
