@@ -155,11 +155,12 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                         LOG_INFO, 
                         "Error: HY024" );
         
+                function_entry( connection );
                 __post_internal_error( &connection -> error,
                     ERROR_HY024, NULL,
                     connection -> environment -> requested_version );
         
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             else 
             {
@@ -223,12 +224,13 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                             LOG_INFO, 
                             LOG_INFO, 
                             "Error: HY024" );
-            
+
+                    function_entry( connection );
                     __post_internal_error( &connection -> error,
                         ERROR_HY024, NULL,
                         connection -> environment -> requested_version );
             
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
                 else 
                 {
@@ -253,13 +255,14 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                         __LINE__, 
                         LOG_INFO, 
                         LOG_INFO, 
-                        "Error: HY009" );
-        
+                        "Error: HY024" );
+
+                function_entry( connection );
                 __post_internal_error( &connection -> error,
-                    ERROR_HY009, NULL,
+                    ERROR_HY024, NULL,
                     connection -> environment -> requested_version );
         
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             else 
             {
@@ -352,7 +355,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_08003, NULL,
                 connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
     else if ( connection -> state == STATE_C3 )
@@ -367,7 +370,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
             ERROR_HY010, NULL,
             connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
     else if ( connection -> state == STATE_C4 ||
             connection -> state == STATE_C5 || 
@@ -385,7 +388,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
         else if ( attribute == SQL_ATTR_PACKET_SIZE )
         {
@@ -399,7 +402,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_HY011, NULL,
                 connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -420,7 +423,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_HY024, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -429,10 +432,34 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
 
     switch( attribute ) 
     {
+        /* ODBC 3.x statement attributes are not settable at the connection level */
+        case SQL_ATTR_APP_PARAM_DESC:
+        case SQL_ATTR_APP_ROW_DESC:
+        case SQL_ATTR_CURSOR_SCROLLABLE:
+        case SQL_ATTR_CURSOR_SENSITIVITY:
+        case SQL_ATTR_ENABLE_AUTO_IPD:
+        case SQL_ATTR_FETCH_BOOKMARK_PTR:
+        case SQL_ATTR_IMP_PARAM_DESC:
+        case SQL_ATTR_IMP_ROW_DESC:
+        case SQL_ATTR_PARAM_BIND_OFFSET_PTR:
+        case SQL_ATTR_PARAM_BIND_TYPE:
+        case SQL_ATTR_PARAM_OPERATION_PTR:
+        case SQL_ATTR_PARAM_STATUS_PTR:
+        case SQL_ATTR_PARAMS_PROCESSED_PTR:
+        case SQL_ATTR_PARAMSET_SIZE:
+        case SQL_ATTR_ROW_ARRAY_SIZE:
+        case SQL_ATTR_ROW_BIND_OFFSET_PTR:
+        case SQL_ATTR_ROW_OPERATION_PTR:
+        case SQL_ATTR_ROW_STATUS_PTR:
+        case SQL_ATTR_ROWS_FETCHED_PTR:
+            __post_internal_error( &connection -> error,
+                    ERROR_HY092, NULL,
+                    connection -> environment -> requested_version );
+
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
+
       	case SQL_ATTR_CONCURRENCY:
 	  	case SQL_BIND_TYPE:
-      	case SQL_ATTR_CURSOR_SCROLLABLE:
-      	case SQL_ATTR_CURSOR_SENSITIVITY:
       	case SQL_ATTR_CURSOR_TYPE:
       	case SQL_ATTR_MAX_LENGTH:
       	case SQL_MAX_ROWS:
@@ -455,28 +482,11 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                         ERROR_24000, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             break;
 
         default:
-            if ( attribute == SQL_ATTR_CURRENT_CATALOG ) 
-            {
-                if( __check_stmt_from_dbc_v( connection, 3, STATE_S5, STATE_S6, STATE_S7 )) {
-
-                    dm_log_write( __FILE__, 
-                            __LINE__, 
-                            LOG_INFO, 
-                            LOG_INFO, 
-                            "Error: 24000" );
-
-                    __post_internal_error( &connection -> error,
-                            ERROR_24000, NULL,
-                            connection -> environment -> requested_version );
-
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
-                }
-            }
 
             if( __check_stmt_from_dbc_v( connection, 8, STATE_S8, STATE_S9, STATE_S10, STATE_S11, STATE_S12, STATE_S13, STATE_S14, STATE_S15 )) {
 
@@ -490,7 +500,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                         ERROR_HY010, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             break;
     }
@@ -570,7 +580,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
             connection -> txn_isolation = ( SQLLEN ) value;
             connection -> txn_isolation_set = 1;
         }
-        else
+        else if ( attribute != SQL_ATTR_LOGIN_TIMEOUT )
         {
             /*
              * save any unknown attributes untill connect
@@ -587,12 +597,23 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
             }
             else if ( string_length == SQL_NTS )
             {
-                sa -> str_attr = strdup( value );
-                sa -> str_len = string_length;
+                if (!value)
+                {
+                    __post_internal_error( &connection -> error,
+                        ERROR_HY024, "Invalid argument value",
+                        connection -> environment -> requested_version );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
+                else
+                {
+                    sa -> str_attr = unicode_to_ansi_alloc( value, string_length, connection, NULL );
+                    sa -> str_len = string_length;
+                }
             }
             else
             {
-                sa -> int_attr = ( SQLLEN ) value;
+                sa -> intptr_attr = (intptr_t) value;
+                sa -> str_len = string_length;
             }
             sa -> next = connection -> save_attr;
             connection -> save_attr = sa;
@@ -608,7 +629,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
     }
     else
     {
@@ -636,7 +657,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     ret = SQLSETCONNECTOPTIONW( connection,
@@ -656,7 +677,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -693,7 +714,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     switch( attribute )
@@ -727,7 +748,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -797,5 +818,5 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
         connection -> bookmarks_on = (SQLULEN) value;
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret );
+    return function_return( SQL_HANDLE_DBC, connection, ret, DEFER_R3 );
 }
