@@ -294,7 +294,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                  ERROR_HY011, NULL,
                  statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
         else if ( statement -> state == STATE_S4 ||
                 statement -> state == STATE_S5 ||
@@ -311,7 +311,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                  ERROR_24000, NULL,
                  statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
         else if ( statement -> state == STATE_S8 ||
                 statement -> state == STATE_S9 ||
@@ -334,7 +334,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                      ERROR_HY011, NULL,
                      statement -> connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
             else
             {
@@ -348,7 +348,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                      ERROR_HY010, NULL,
                      statement -> connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
         }
     }
@@ -370,7 +370,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                  ERROR_HY010, NULL,
                  statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
@@ -388,7 +388,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -455,7 +455,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 			statement -> msg );
     			}
 			
-    			return function_return( SQL_HANDLE_STMT, statement, ret ); 
+    			return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 			}
 			
 			/*
@@ -477,7 +477,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 		statement -> msg );
     		}
 		
-    		return function_return( SQL_HANDLE_STMT, statement, ret ); 
+    		return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 		}
 
         if ( !__validate_desc( desc ))
@@ -510,7 +510,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY017, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         if ( desc -> connection !=
@@ -526,7 +526,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY024, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         /*
@@ -597,7 +597,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 			statement -> msg );
     			}
 			
-    			return function_return( SQL_HANDLE_STMT, statement, ret ); 
+    			return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 			}
 			
 			/*
@@ -619,7 +619,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 		statement -> msg );
     		}
 		
-    		return function_return( SQL_HANDLE_STMT, statement, ret ); 
+    		return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 		}
 
         if ( !__validate_desc( desc ))
@@ -652,7 +652,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY017, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         if ( desc -> connection !=
@@ -668,7 +668,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY024, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         /*
@@ -701,7 +701,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 ERROR_HY017, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -721,7 +721,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 ERROR_HY024, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -900,7 +900,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY092, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         ret = SQLSETSTMTOPTION( statement -> connection,
@@ -931,5 +931,5 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret ); 
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 }
