@@ -3,7 +3,7 @@
  * unixODBC Cursor Library
  *
  * Created by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * copyright (c) 1999 Nick Gorham
  *
@@ -23,9 +23,44 @@
  *
  **********************************************************************
  *
- * $Id: SQLExtendedFetch.c,v 1.3 2002/11/25 15:37:54 lurcher Exp $
+ * $Id: SQLExtendedFetch.c,v 1.14 2009/02/18 17:59:17 lurcher Exp $
  *
  * $Log: SQLExtendedFetch.c,v $
+ * Revision 1.14  2009/02/18 17:59:17  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.13  2009/02/17 09:47:45  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.12  2008/01/22 17:51:54  lurcher
+ * Another SQLULEN mismatch
+ *
+ * Revision 1.11  2007/11/29 12:00:36  lurcher
+ * Add 64 bit type changes to SQLExtendedFetch etc
+ *
+ * Revision 1.10  2007/11/13 15:04:57  lurcher
+ * Fix 64 bit cursor lib issues
+ *
+ * Revision 1.9  2005/10/27 17:54:49  lurcher
+ * fix what I suspect is a typo in qt.m4
+ *
+ * Revision 1.8  2005/10/21 16:49:53  lurcher
+ * Fix a problem with the cursor lib and rowsets
+ *
+ * Revision 1.7  2004/07/24 17:55:38  lurcher
+ * Sync up CVS
+ *
+ * Revision 1.6  2003/12/01 16:37:17  lurcher
+ *
+ * Fix a bug in SQLWritePrivateProfileString
+ *
+ * Revision 1.5  2003/10/30 18:20:46  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
  * Revision 1.3  2002/11/25 15:37:54  lurcher
  *
  * Fix problems in the cursor lib
@@ -65,10 +100,13 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "cursorlibrary.h"
 
 #define ABS(x)  (((x)>=0)?(x):(-(x)))
 
+#define SQL_FETCH_PART_ROWSET SQL_NO_DATA + 1
+
 SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
 {
     SQLSMALLINT ret;
@@ -173,7 +211,7 @@ SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
                     if ( cbuf -> bound_ind )
                     {
                         ind_ptr = ( char * ) cbuf -> bound_ind;
-                        ind_ptr += offset * sizeof( SQLUINTEGER );
+                        ind_ptr += offset * sizeof( SQLULEN );
                     }
                 }
 
@@ -291,11 +329,17 @@ SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
                         if ( cbuf -> bound_ind )
                         {
                             ind_ptr = ( char * ) cbuf -> bound_ind;
-                            ind_ptr += offset * sizeof( SQLUINTEGER );
+                            ind_ptr += offset * sizeof( SQLULEN );
                         }
                     }
-                    
+
+                    /*
+                     * Not quite sure if the check is valid, I think I can see where
+                     * I got it from, but not sure if it's correct
+                     * 
                     if ( buffer && cbuf -> bound_ind && *cbuf -> bound_ind >= 0 )
+                    */
+                    if ( buffer && cbuf -> bound_ind )
                     {
                         if ( cbuf -> bound_type == SQL_C_CHAR )
                         {
@@ -425,7 +469,7 @@ static SQLRETURN fetch_rowset( CLHSTMT cl_statement,
         int row_offset,
         int *fetched_rows,
         SQLUSMALLINT *row_status_array,
-        SQLUINTEGER *rows_fetched_ptr )
+        SQLULEN *rows_fetched_ptr )
 {
     SQLRETURN ret;
     int row_count = 0;
@@ -451,6 +495,17 @@ static SQLRETURN fetch_rowset( CLHSTMT cl_statement,
         }
         ret = SQL_SUCCESS;
     }
+
+    if ( ret == SQL_NO_DATA && row > 0 )
+	{
+        *fetched_rows = row;
+    	if ( rows_fetched_ptr )
+		{
+        	*rows_fetched_ptr = row_count;
+		}
+		ret = SQL_FETCH_PART_ROWSET;
+	}
+
     if ( SQL_SUCCEEDED( ret ))
     {
         *fetched_rows = row;
@@ -466,13 +521,16 @@ static SQLRETURN fetch_rowset( CLHSTMT cl_statement,
 
 SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
         int fetch_orientation,
-        int fetch_offset,
+        SQLLEN fetch_offset,
         SQLUSMALLINT *row_status_ptr,
-        SQLUINTEGER *rows_fetched_ptr )
+        SQLULEN *rows_fetched_ptr,
+		int ext_fetch )
 {
     SQLRETURN ret;
     int rows_in_set, row_offset, fetched_rows, info = 0;
 
+	cl_statement -> fetch_done = 1;
+
     if ( !cl_statement -> first_fetch_done )
     {
         if ( cl_statement -> column_count > 0 && 
@@ -492,10 +550,20 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
         cl_statement -> first_fetch_done = 1;
     }
 
-    if ( cl_statement -> rowset_array_size < 1 )
-        rows_in_set = 1;
-    else 
-        rows_in_set = cl_statement -> rowset_array_size;
+	if ( ext_fetch ) 
+	{
+    	if ( cl_statement -> rowset_size < 1 )
+       	 	rows_in_set = 1;
+    	else 
+        	rows_in_set = cl_statement -> rowset_size;
+	}
+	else 
+	{
+    	if ( cl_statement -> rowset_array_size < 1 )
+       	 	rows_in_set = 1;
+    	else 
+        	rows_in_set = cl_statement -> rowset_array_size;
+	}
 
     /*
      * I refuse to document all these conditions, you have to
@@ -516,6 +584,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
         break;
 
       case SQL_FETCH_NEXT:
@@ -545,6 +617,11 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+        	cl_statement -> rowset_position = CL_AFTER_END;
+			ret = SQL_SUCCESS;
+		}
         break;
 
       case SQL_FETCH_PRIOR:
@@ -553,21 +630,6 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
             ret = SQL_NO_DATA;
             break;
         }
-        else if ( cl_statement -> rowset_position <= rows_in_set  )
-        {
-            ret = SQL_NO_DATA;
-            cl_statement -> rowset_position = CL_BEFORE_START;
-            break;
-        }
-        else if ( cl_statement -> rowset_position - rows_in_set < rows_in_set )
-        {
-            cl_statement -> cl_connection -> dh.__post_internal_error( &cl_statement -> dm_statement -> error,
-                    ERROR_01S06, NULL,
-                    cl_statement -> dm_statement -> connection ->
-                        environment -> requested_version );
-            ret = SQL_SUCCESS_WITH_INFO;
-            break;
-        }
         else if ( cl_statement -> rowset_position == CL_AFTER_END )
         {
             if ( cl_statement -> rowset_count < rows_in_set )
@@ -581,8 +643,24 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
             else
             {
                 row_offset = cl_statement -> rowset_count - rows_in_set;
+	    	cl_statement -> rowset_position = row_offset;
             }
         }
+        else if ( cl_statement -> rowset_position <= rows_in_set  )
+        {
+            ret = SQL_NO_DATA;
+            cl_statement -> rowset_position = CL_BEFORE_START;
+            break;
+        }
+        else if ( cl_statement -> rowset_position - rows_in_set < rows_in_set )
+        {
+            cl_statement -> cl_connection -> dh.__post_internal_error( &cl_statement -> dm_statement -> error,
+                    ERROR_01S06, NULL,
+                    cl_statement -> dm_statement -> connection ->
+                        environment -> requested_version );
+            ret = SQL_SUCCESS_WITH_INFO;
+            break;
+        }
         else
         {
             row_offset = cl_statement -> rowset_position -= rows_in_set * 2;
@@ -599,6 +677,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
         break;
 
       case SQL_FETCH_RELATIVE:
@@ -611,7 +693,8 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                     SQL_FETCH_ABSOLUTE,
                     fetch_offset,
                     row_status_ptr,
-                    rows_fetched_ptr );
+                    rows_fetched_ptr, 
+					ext_fetch );
         }
         else if ( cl_statement -> rowset_position == CL_BEFORE_START &&
             fetch_offset <= 0 )
@@ -688,9 +771,17 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
 
         break;
 
+      /*
+       * The code before this turns the bookmark into a absolute
+       */
+      case SQL_FETCH_BOOKMARK:
       case SQL_FETCH_ABSOLUTE:
         /*
          * close the rowset if needed
@@ -776,6 +867,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
 
         break;
 
@@ -812,11 +907,12 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
             cl_statement -> curr_rowset_start = 
                 cl_statement -> rowset_position = CL_AFTER_END;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
         
         break;
-
-      case SQL_FETCH_BOOKMARK:
-        break;
     }
 
     if ( ret == SQL_SUCCESS && info )
@@ -828,8 +924,8 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
 SQLRETURN CLExtendedFetch(
     SQLHSTMT           statement_handle,
     SQLUSMALLINT       f_fetch_type,
-    SQLINTEGER         irow,
-    SQLUINTEGER        *pcrow,
+    SQLLEN             irow,
+    SQLULEN            *pcrow,
     SQLUSMALLINT       *rgf_row_status )
 {
     CLHSTMT cl_statement = (CLHSTMT) statement_handle; 
@@ -848,5 +944,6 @@ SQLRETURN CLExtendedFetch(
             f_fetch_type, 
             irow,
             rgf_row_status,
-            pcrow );
+            pcrow,
+			1 );
 }
