@@ -19,12 +19,14 @@
 char *szSyntax =
 "\n" \
 "**********************************************\n" \
-"* unixODBC - isql                            *\n" \
+"* unixODBC - isql and iusql                  *\n" \
 "**********************************************\n" \
 "* Syntax                                     *\n" \
 "*                                            *\n" \
 "*      isql DSN [UID [PWD]] [options]        *\n" \
 "*                                            *\n" \
+"*      iusql DSN [UID [PWD]] [options]       *\n" \
+"*                                            *\n" \
 "* Options                                    *\n" \
 "*                                            *\n" \
 "* -b         batch.(no prompting etc)        *\n" \
@@ -53,13 +55,17 @@ char *szSyntax =
 "*                                            *\n" \
 "* Examples                                   *\n" \
 "*                                            *\n" \
-"*      isql WebDB MyID MyPWD -w < My.sql     *\n" \
+"*      iusql -v WebDB MyID MyPWD -w < My.sql *\n" \
 "*                                            *\n" \
 "*      Each line in My.sql must contain      *\n" \
 "*      exactly 1 SQL command except for the  *\n" \
 "*      last line which must be blank (unless *\n" \
 "*      -n option specified).                 *\n" \
 "*                                            *\n" \
+"* Datasources, drivers, etc:                 *\n" \
+"*                                            *\n" \
+"*      See \"man 1 isql\"                    *\n" \
+"*                                            *\n" \
 "* Please visit;                              *\n" \
 "*                                            *\n" \
 "*      http://www.unixodbc.org               *\n" \
@@ -72,12 +78,14 @@ char *szSyntax =
 char *szSyntax =
 "\n" \
 "**********************************************\n" \
-"* unixODBC - isql                            *\n" \
+"* unixODBC - isql and iusql                  *\n" \
 "**********************************************\n" \
 "* Syntax                                     *\n" \
 "*                                            *\n" \
 "*      isql DSN [UID [PWD]] [options]        *\n" \
 "*                                            *\n" \
+"*      iusql DSN [UID [PWD]] [options]       *\n" \
+"*                                            *\n" \
 "* Options                                    *\n" \
 "*                                            *\n" \
 "* -b         batch.(no prompting etc)        *\n" \
@@ -91,7 +99,6 @@ char *szSyntax =
 "* -v         verbose.                        *\n" \
 "* -q         wrap char fields in dquotes     *\n" \
 "* --version  version                         *\n" \
-"* --version  version                         *\n" \
 "*                                            *\n" \
 "* Commands                                   *\n" \
 "*                                            *\n" \
@@ -101,12 +108,16 @@ char *szSyntax =
 "*                                            *\n" \
 "* Examples                                   *\n" \
 "*                                            *\n" \
-"*      isql WebDB MyID MyPWD -w < My.sql     *\n" \
+"*      iusql -v WebDB MyID MyPWD -w < My.sql *\n" \
 "*                                            *\n" \
 "*      Each line in My.sql must contain      *\n" \
 "*      exactly 1 SQL command except for the  *\n" \
 "*      last line which must be blank.        *\n" \
 "*                                            *\n" \
+"* Datasources, drivers, etc:                 *\n" \
+"*                                            *\n" \
+"*      See \"man 1 isql\"                    *\n" \
+"*                                            *\n" \
 "* Please visit;                              *\n" \
 "*                                            *\n" \
 "*      http://www.unixodbc.org               *\n" \
