@@ -9,6 +9,7 @@
  * -----------------------------------------------
  * Peter Harvey		- pharvey@codebydesign.com
  **************************************************/
+#include <config.h>
 #include <odbcinstext.h>
 
 
@@ -36,9 +37,11 @@ char *szSyntax =
 "*      -u         uninstall                  *\n" \
 "*      -q         query                      *\n" \
 "*      -j         print config info          *\n" \
+"*      -c         call SQLCreateDataSource   *\n" \
+"*      -m         call SQLManageDataSources  *\n" \
 "*      --version  version                    *\n" \
 "*                                            *\n" \
-"* Objects:                                   *\n" \
+"* Object:                                    *\n" \
 "*                                            *\n" \
 "*      -d driver                             *\n" \
 "*      -s data source                        *\n" \
@@ -73,6 +76,68 @@ int from_stdin = 0;
 int system_dsn = 0;
 int user_dsn = 0;
 
+/*!
+ * \brief   Invoke UI to Create Data Source wizard.
+ *
+ *          This exists so we can test calling SQLCreateDataSource from an
+ *          app which does not provide the UI used by SQLCreateDataSource.
+ *
+ *          At the moment we have "odbcinstQ4" (the Qt4 based UI) requested
+ *          explicitly but this could be changed to simply request the default
+ *          or to use a ncurses based UI when that becomes available.
+ *
+ *          There are at least 3 ways to invoke SQLCreateDataSource;
+ *
+ *          \li ODBCCreateDataSourceQ4 at the command-line
+ *          \li a custom application
+ *          \li "odbcinst -c [-nMyDsn]" at the command-line
+ *
+ * \sa      ManageDataSources
+ */
+int CreateDataSource( char *pszDataSourceName )
+{
+    ODBCINSTWND odbcinstwnd;
+
+    odbcinstwnd.hWnd = 0;
+    strcpy( odbcinstwnd.szUI, ODBCINSTPLUGIN );
+
+    if ( SQLCreateDataSource( (HWND)&(odbcinstwnd), ( (pszDataSourceName && *pszDataSourceName) ? pszDataSourceName : 0 ) ) == FALSE )
+        return 1;
+
+    return 0;
+}
+
+/*!
+ * \brief   Invoke UI to Manage Data Sources.
+ *
+ *          This exists so we can test calling SQLManageDataSources from an
+ *          app which does not provide the UI used by SQLManageDataSources.
+ *
+ *          At the moment we have "odbcinstQ4" (the Qt4 based UI) requested
+ *          explicitly but this could be changed to simply request the default
+ *          or to use a ncurses based UI when that becomes available.
+ *
+ *          There are at least 3 ways to invoke SQLManageDataSources;
+ *
+ *          \li ODBCManageDataSourcesQ4 at the command-line
+ *          \li a custom application
+ *          \li "odbcinst -m" at the command-line
+ *
+ * \sa      CreateDataSource
+ */
+int ManageDataSources()
+{
+    ODBCINSTWND odbcinstwnd;
+
+    odbcinstwnd.hWnd = 0;
+    strcpy( odbcinstwnd.szUI, ODBCINSTPLUGIN );
+
+    if ( SQLManageDataSources( (HWND)&(odbcinstwnd) ) == FALSE )
+        return 1;
+
+    return 0;
+}
+
 int DriverInstall( char *pszTemplate )
 {
     HINI    hIni;
@@ -84,7 +149,11 @@ int DriverInstall( char *pszTemplate )
     DWORD   nUsageCount = 0;
     char    *pChar      = NULL;
 
-    if ( iniOpen( &hIni, pszTemplate, '#', '[', ']', '=', FALSE ) != INI_SUCCESS )
+#ifdef __OS2__
+    if ( iniOpen( &hIni, pszTemplate, "#;", '[', ']', '=', FALSE, 0L ) != INI_SUCCESS )
+#else
+    if ( iniOpen( &hIni, pszTemplate, "#;", '[', ']', '=', FALSE ) != INI_SUCCESS )
+#endif
     {
         if ( cVerbose == 0 ) printf( "odbcinst: iniOpen failed on %s.\n", pszTemplate );
         return 1;
@@ -115,7 +184,7 @@ int DriverInstall( char *pszTemplate )
             if ( cVerbose == 0 ) printf( "odbcinst: SQLInstallDriverEx failed with %s.\n", szError );
             return 1;
         }
-        if ( cVerbose == 0 ) printf( "odbcinst: Driver installed. Usage count increased to %d. Target directory is %s\n", (int)nUsageCount, szPathOut );
+        if ( cVerbose == 0 ) printf( "odbcinst: Driver installed. Usage count increased to %d. \n    Target directory is %s\n", (int)nUsageCount, szPathOut );
         memset( szDriver, '\0', 10000 );
         pChar = szDriver;
         iniObjectNext( hIni );
@@ -151,60 +220,47 @@ int DriverUninstall( char *pszDriver )
 
 int DriverQuery( char *pszDriver )
 {
-    int     nElement;
     char    szResults[4048];
-/*    char    szDriverName[INI_MAX_OBJECT_NAME+1]; */
     char    szValue[501];
-    char    szElement[INI_MAX_OBJECT_NAME+1];
+    char *ptr;
 
-    if ( pszDriver && (*pszDriver) ) 
+    if ( pszDriver && (*pszDriver) )
     {
         /* list Driver details */
-        if ( SQLGetPrivateProfileString( pszDriver, NULL, NULL, szResults, 9600, "ODBCINST.INI" ) < 1 )
+        if ( SQLGetPrivateProfileString( pszDriver, NULL, NULL, szResults, sizeof( szResults ) - 1, "ODBCINST.INI" ) < 1 )
         {
             SQLInstallerError( 1, &nError, szError, ODBC_FILENAME_MAX, NULL );
             if ( cVerbose == 0 ) printf( "odbcinst: SQLGetPrivateProfileString failed with %s.\n", szError );
             return 1;
         }
         printf( "[%s]\n", pszDriver );
-        for ( nElement = 0; iniElement( szResults, '\0', '\0', nElement, szElement, INI_MAX_OBJECT_NAME ) == INI_SUCCESS; nElement++ )
+        ptr = szResults;
+        while ( *ptr )
         {
-            printf( "%s=", szElement );
-            if ( SQLGetPrivateProfileString( pszDriver, szElement, "", szValue, 500, "ODBCINST.INI" ) > 0 )
+            printf( "%s=", ptr );
+            if ( SQLGetPrivateProfileString( pszDriver, ptr, "", szValue, sizeof( szValue ) - 1, "ODBCINST.INI" ) > 0 )
             {
                 printf( "%s", szValue );
             }
             printf( "\n" );
+            ptr += strlen( ptr ) + 1;
         }
     }
     else
     {
         /* list Drivers */
-        if ( SQLGetPrivateProfileString( NULL, NULL, NULL, szResults, 9600, "ODBCINST.INI" ) < 1 )
+        if ( SQLGetPrivateProfileString( NULL, NULL, NULL, szResults, sizeof( szResults ) - 1, "ODBCINST.INI" ) < 1 )
         {
             SQLInstallerError( 1, &nError, szError, ODBC_FILENAME_MAX, NULL );
             if ( cVerbose == 0 ) printf( "odbcinst: SQLGetPrivateProfileString failed with %s.\n", szError );
             return 1;
         }
-        for ( nElement = 0; iniElement( szResults, '\0', '\0', nElement, szElement, INI_MAX_OBJECT_NAME ) == INI_SUCCESS; nElement++ )
+        ptr = szResults;
+        while ( *ptr )
         {
-            printf( "[%s]\n", szElement );
+            printf( "[%s]\n", ptr );
+            ptr += strlen( ptr ) + 1;
         }
-
-/*
-        szResults[0] = '\0';
-        if ( SQLGetInstalledDrivers( szResults, 4048, NULL ) == FALSE )
-        {
-            SQLInstallerError( 1, &nError, szError, ODBC_FILENAME_MAX, NULL );
-            if ( cVerbose == 0 ) printf( "odbcinst: SQLGetInstalledDrivers failed with %s.\n", szError );
-            return 1;
-        }
-
-        for ( nElement = 0; iniElement( szResults, '\0', '\0', nElement, szDriverName, INI_MAX_OBJECT_NAME ) == INI_SUCCESS; nElement++ )
-        {
-            printf( "[%s]\n", szDriverName );
-        }
-*/        
     }
 
     return 0;
@@ -218,7 +274,11 @@ int DSNInstall( char *pszTemplate )
     char    szProperty[INI_MAX_PROPERTY_NAME+1];
     char    szValue[INI_MAX_PROPERTY_VALUE+1];
 
-    if ( iniOpen( &hIni, pszTemplate, '#', '[', ']', '=', FALSE ) != INI_SUCCESS )
+#ifdef __OS2__
+    if ( iniOpen( &hIni, pszTemplate, "#;", '[', ']', '=', FALSE, 0L ) != INI_SUCCESS )
+#else
+    if ( iniOpen( &hIni, pszTemplate, "#;", '[', ']', '=', FALSE ) != INI_SUCCESS )
+#endif
     {
         if ( cVerbose == 0 ) printf( "odbcinst: iniOpen failed on %s.\n", pszTemplate );
         return 1;
@@ -291,6 +351,15 @@ int DSNUninstall( char *pszDSN )
     UWORD   nConfigMode;
     char    *pMode;
 
+    if ( system_dsn )
+    {
+        SQLSetConfigMode( ODBC_SYSTEM_DSN );
+    }
+    else if ( user_dsn )
+    {
+        SQLSetConfigMode( ODBC_USER_DSN );
+    }
+
     if ( SQLGetConfigMode( &nConfigMode ) == FALSE )
     {
         SQLInstallerError( 1, &nError, szError, ODBC_FILENAME_MAX, NULL );
@@ -306,17 +375,17 @@ int DSNUninstall( char *pszDSN )
 
     switch ( nConfigMode )
     {
-    case ODBC_SYSTEM_DSN:
-        pMode = "ODBC_SYSTEM_DSN";
-        break;
-    case ODBC_USER_DSN:
-        pMode = "ODBC_USER_DSN";
-        break;
-    case ODBC_BOTH_DSN:
-        pMode = "ODBC_BOTH_DSN";
-        break;
-    default:
-        pMode = "Unknown mode";
+        case ODBC_SYSTEM_DSN:
+            pMode = "ODBC_SYSTEM_DSN";
+            break;
+        case ODBC_USER_DSN:
+            pMode = "ODBC_USER_DSN";
+            break;
+        case ODBC_BOTH_DSN:
+            pMode = "ODBC_BOTH_DSN";
+            break;
+        default:
+            pMode = "Unknown mode";
     }
     if ( cVerbose == 0 ) printf( "odbcinst: DSN removed (if it existed at all). %s was used as the search path.\n", pMode );
 
@@ -325,10 +394,9 @@ int DSNUninstall( char *pszDSN )
 
 int DSNQuery( char *pszDSN )
 {
-    int     nElement;
     char    szResults[9601];
     char    szValue[501];
-    char    szElement[INI_MAX_OBJECT_NAME+1];
+    char    *ptr;
 
     szResults[0] = '\0';
 
@@ -340,7 +408,7 @@ int DSNQuery( char *pszDSN )
     if ( pszDSN && (*pszDSN) )
     {
         /* list DSN details */
-        if ( SQLGetPrivateProfileString( pszDSN, NULL, NULL, szResults, 9600, "ODBC.INI" ) < 1 )
+        if ( SQLGetPrivateProfileString( pszDSN, NULL, NULL, szResults, sizeof( szResults ) - 1, "ODBC.INI" ) < 1 )
         {
             SQLInstallerError( 1, &nError, szError, ODBC_FILENAME_MAX, NULL );
             if ( cVerbose == 0 ) printf( "odbcinst: SQLGetPrivateProfileString failed with %s.\n", szError );
@@ -348,29 +416,33 @@ int DSNQuery( char *pszDSN )
             return 1;
         }
         printf( "[%s]\n", pszDSN );
-        for ( nElement = 0; iniElement( szResults, '\0', '\0', nElement, szElement, INI_MAX_OBJECT_NAME ) == INI_SUCCESS; nElement++ )
+        ptr = szResults;
+        while ( *ptr )
         {
-            printf( "%s=", szElement );
-            if ( SQLGetPrivateProfileString( pszDSN, szElement, "", szValue, 500, "ODBC.INI" ) > 0 )
+            printf( "%s=", ptr );
+            if ( SQLGetPrivateProfileString( pszDSN, ptr, "", szValue, sizeof( szValue ) - 1, "ODBC.INI" ) > 0 )
             {
                 printf( "%s", szValue );
             }
             printf( "\n" );
+            ptr += strlen( ptr ) + 1;
         }
     }
     else
     {
         /* list DSNs */
-        if ( SQLGetPrivateProfileString( NULL, NULL, NULL, szResults, 9600, "ODBC.INI" ) < 1 )
+        if ( SQLGetPrivateProfileString( NULL, NULL, NULL, szResults, sizeof( szResults ) - 1, "ODBC.INI" ) < 1 )
         {
             SQLInstallerError( 1, &nError, szError, ODBC_FILENAME_MAX, NULL );
             if ( cVerbose == 0 ) printf( "odbcinst: SQLGetPrivateProfileString failed with %s.\n", szError );
             SQLSetConfigMode( ODBC_BOTH_DSN );
             return 1;
         }
-        for ( nElement = 0; iniElement( szResults, '\0', '\0', nElement, szElement, INI_MAX_OBJECT_NAME ) == INI_SUCCESS; nElement++ )
+        ptr = szResults;
+        while ( *ptr )
         {
-            printf( "[%s]\n", szElement );
+            printf( "[%s]\n", ptr );
+            ptr += strlen( ptr ) + 1;
         }
     }
 
@@ -384,26 +456,35 @@ void Syntax()
     if ( cVerbose != 0 )
         return;
 
-    printf( szSyntax );
+    puts( szSyntax );
 }
 
 void PrintConfigInfo()
 {
     char szFileName[ODBC_FILENAME_MAX+1];
+	char b1[ 256 ], b2[ 256 ];
 
     printf( "unixODBC " VERSION "\n" );
 
     *szFileName = '\0';
-    sprintf( szFileName, "%s/odbcinst.ini", odbcinst_system_file_path() );
+    sprintf( szFileName, "%s/%s", odbcinst_system_file_path( b1 ), odbcinst_system_file_name( b2 ));
     printf( "DRIVERS............: %s\n", szFileName ); 
 
     *szFileName = '\0';
     _odbcinst_SystemINI( szFileName, FALSE );
     printf( "SYSTEM DATA SOURCES: %s\n", szFileName ); 
 
+    *szFileName = '\0';
+    _odbcinst_FileINI( szFileName );
+    printf( "FILE DATA SOURCES..: %s\n", szFileName ); 
+
     *szFileName = '\0';
     _odbcinst_UserINI( szFileName, FALSE );
     printf( "USER DATA SOURCES..: %s\n", szFileName ); 
+
+	printf( "SQLULEN Size.......: %ld\n", (long) sizeof( SQLULEN )); 
+	printf( "SQLLEN Size........: %ld\n", (long) sizeof( SQLLEN )); 
+	printf( "SQLSETPOSIROW Size.: %ld\n", (long) sizeof( SQLSETPOSIROW )); 
 }
 
 int main( int argc, char *argv[] )
@@ -431,57 +512,59 @@ int main( int argc, char *argv[] )
         {
             switch ( argv[nArg][1] )
             {
-            /* Action */
-            case 'i':
-            case 'u':
-            case 'q':
-                cAction = argv[nArg][1];
-                break;
-            case 'j':
-                PrintConfigInfo();
-                exit(0);
-            case '-':
-                printf( "unixODBC " VERSION "\n" );
-                exit(0);
-                /* Object */
-            case 'd':
-            case 's':
-                cObject = argv[nArg][1];
-                break;
-                /* Options */
-            case 'n':
-                if ( nArg < argc-1 )
-                    strncpy( szObjectName, argv[nArg+1], INI_MAX_OBJECT_NAME );
-                break;
-            case 'f':
-                if ( nArg < argc-1 )
-                    strncpy( szTemplateINI, argv[nArg+1], ODBC_FILENAME_MAX );
-                break;
-            case 'r':
-                from_stdin = 1;
-                break;
-            case 'v':
-                cVerbose = argv[nArg][1];
-                break;
-            case 'l':
-                system_dsn = 1;
-                if ( user_dsn )
-                {
-                    if ( cVerbose == 0 ) printf( "odbcinst: cannot install both user and system dsn at the same time");
-                    exit( -2 );
-                }
-                break;
-            case 'h':
-                user_dsn = 1;
-                if ( system_dsn )
-                {
-                    if ( cVerbose == 0 ) printf( "odbcinst: cannot install both user and system dsn at the same time");
-                    exit( -2 );
-                }
-                break;
-            default:
-                if ( cVerbose == 0 ) printf( "odbcinst: Unknown option %c\n", argv[nArg][1] );
-                exit( -1 );
+                /* Action */
+                case 'i':
+                case 'u':
+                case 'q':
+                    cAction = argv[nArg][1];
+                    break;
+                case 'j':
+                    PrintConfigInfo();
+                    exit(0);
+                case '-':
+                    printf( "unixODBC " VERSION "\n" );
+                    exit(0);
+                    /* Object */
+                case 'c':
+                case 'd':
+                case 's':
+                case 'm':
+                    cObject = argv[nArg][1];
+                    break;
+                    /* Options */
+                case 'n':
+                    if ( nArg < argc-1 )
+                        strncpy( szObjectName, argv[nArg+1], INI_MAX_OBJECT_NAME );
+                    break;
+                case 'f':
+                    if ( nArg < argc-1 )
+                        strncpy( szTemplateINI, argv[nArg+1], ODBC_FILENAME_MAX );
+                    break;
+                case 'r':
+                    from_stdin = 1;
+                    break;
+                case 'v':
+                    cVerbose = argv[nArg][1];
+                    break;
+                case 'l':
+                    system_dsn = 1;
+                    if ( user_dsn )
+                    {
+                        if ( cVerbose == 0 ) printf( "odbcinst: cannot install both user and system dsn at the same time");
+                        exit( -2 );
+                    }
+                    break;
+                case 'h':
+                    user_dsn = 1;
+                    if ( system_dsn )
+                    {
+                        if ( cVerbose == 0 ) printf( "odbcinst: cannot install both user and system dsn at the same time");
+                        exit( -2 );
+                    }
+                    break;
+                default:
+                    if ( cVerbose == 0 ) printf( "odbcinst: Unknown option %c\n", argv[nArg][1] );
+                    exit( -1 );
             }
         }
     }
@@ -498,7 +581,7 @@ int main( int argc, char *argv[] )
                 nReturn = DriverInstall( STDINFILE );
             else
             {
-                if ( cVerbose == 0 ) printf( "odbcinst: Please supply -f template.ini \n" );
+                if ( cVerbose == 0 ) printf( "odbcinst: Please supply -f template.ini (The fileformat of template.ini is identical to odbcinst.ini and odbc.ini, respectively)\n" );
                 Syntax();
                 exit( 1 );
             }
@@ -566,6 +649,14 @@ int main( int argc, char *argv[] )
         }
 
     }
+    else if ( cObject == 'c' )
+    {
+        nReturn = CreateDataSource( szObjectName );
+    }
+    else if ( cObject == 'm' )
+    {
+        nReturn = ManageDataSources();
+    }
     else
     {
         if ( cVerbose == 0 ) printf( "odbcinst: Invalid Object\n" );
