@@ -182,12 +182,12 @@ static int _check_ini_cache( int *ret,
         if ( pRetBuffer && !ini_cache -> value )
             continue;
 
-        if ( nRetBuffer <= ini_cache -> buffer_size )
+        if ( nRetBuffer < ini_cache -> buffer_size )
             continue;
 
         if ( pRetBuffer )
         {
-            if ( ini_cache -> value )
+            if ( ini_cache -> value ) {
                 if ( nRetBuffer < strlen( ini_cache -> value )) {
                     strncpy( pRetBuffer, ini_cache -> value, nRetBuffer );
                     pRetBuffer[ nRetBuffer - 1 ] = '\0';
@@ -195,6 +195,7 @@ static int _check_ini_cache( int *ret,
                 else {
                     strcpy( pRetBuffer, ini_cache -> value );
                 }
+            }
 
             *ret = ini_cache -> ret_value;
             return 1;
@@ -204,7 +205,7 @@ static int _check_ini_cache( int *ret,
     return 0;
 }
 
-static int _save_ini_cache( int ret,
+static void _save_ini_cache( int ret,
                     LPCSTR  pszSection,
                     LPCSTR  pszEntry,
                     LPCSTR  pszDefault,
@@ -219,7 +220,7 @@ static int _save_ini_cache( int ret,
     ini_cache = calloc( sizeof( struct ini_cache ), 1 );
     if ( !ini_cache )
     {
-        return 0;
+        return;
     }
 
     if ( pszFileName )
@@ -247,12 +248,37 @@ static int _save_ini_cache( int ret,
 
     ini_cache -> next = ini_cache_head;
     ini_cache_head = ini_cache;
+}
 
-    return 0;
+static void _clear_ini_cache( void ) 
+{
+    struct ini_cache *ini_cache = ini_cache_head;
+
+    while (( ini_cache = ini_cache_head ) != NULL  )
+    {
+        ini_cache_head = ini_cache -> next;
+
+        if ( ini_cache -> fname )
+            free( ini_cache -> fname );
+
+        if ( ini_cache -> section )
+            free( ini_cache -> section );
+
+        if ( ini_cache -> entry )
+            free( ini_cache -> entry );
+
+        if ( ini_cache -> value )
+            free( ini_cache -> value );
+
+        if ( ini_cache -> default_value )
+            free( ini_cache -> default_value );
+
+        free( ini_cache );
+    }
 }
 
 /*
- * wrappers to provide therad safety
+ * wrappers to provide thread safety
  */
 
 static int check_ini_cache( int *ret,
@@ -275,7 +301,7 @@ static int check_ini_cache( int *ret,
 	return rval;
 }
 
-static int save_ini_cache( int ret,
+static void save_ini_cache( int ret,
                     LPCSTR  pszSection,
                     LPCSTR  pszEntry,
                     LPCSTR  pszDefault,
@@ -283,7 +309,7 @@ static int save_ini_cache( int ret,
                     int     nRetBuffer,
                     LPCSTR  pszFileName )
 {
-	int rval, cval;
+	int cval;
 
 	mutex_entry( &mutex_ini );
 
@@ -294,15 +320,21 @@ static int save_ini_cache( int ret,
 	if ( !_check_ini_cache( &cval, pszSection, pszEntry, pszDefault,
 			pRetBuffer, nRetBuffer, pszFileName )) {
 
-		rval = _save_ini_cache( ret, pszSection, pszEntry, pszDefault,
+		_save_ini_cache( ret, pszSection, pszEntry, pszDefault,
 			pRetBuffer, nRetBuffer, pszFileName );
 	}
 
 	mutex_exit( &mutex_ini );
-
-	return rval;
 }
 
+void __clear_ini_cache( void ) 
+{
+	mutex_entry( &mutex_ini );
+
+    _clear_ini_cache();
+
+	mutex_exit( &mutex_ini );
+}
 
 #else
 
@@ -317,7 +349,7 @@ static int check_ini_cache( int *ret,
     return 0;
 }
 
-static int save_ini_cache( int ret,
+static void save_ini_cache( int ret,
                     LPCSTR  pszSection,
                     LPCSTR  pszEntry,
                     LPCSTR  pszDefault,
@@ -325,7 +357,10 @@ static int save_ini_cache( int ret,
                     int     nRetBuffer,
                     LPCSTR  pszFileName )
 {
-    return 0;
+}
+
+void __clear_ini_cache( void ) 
+{
 }
 
 #endif
@@ -395,6 +430,10 @@ int SQLGetPrivateProfileString( LPCSTR  pszSection,
 
             return ret;
         }
+        else if ( !*pszFileName )
+        {
+            return 0; /* Asking for empty filename returns nothing, not even default */
+        }
     }
 
     /*****************************************************
@@ -504,6 +543,11 @@ int SQLGetPrivateProfileString( LPCSTR  pszSection,
     if ( pszSection == NULL )
     {
         _odbcinst_GetSections( hIni, pRetBuffer, nRetBuffer, &nBufPos );
+
+        if (nBufPos > 0)
+            ret = _multi_string_length(pRetBuffer);
+        else
+            ret = 0;  /* Indicate not found */
     }
     /*****************************************************
      * EXTRACT ENTRIES
@@ -511,6 +555,10 @@ int SQLGetPrivateProfileString( LPCSTR  pszSection,
     else if ( pszEntry == NULL )
     {
         _odbcinst_GetEntries( hIni, pszSection, pRetBuffer, nRetBuffer, &nBufPos );
+        if (nBufPos > 0)
+            ret = _multi_string_length(pRetBuffer);
+        else
+            ret = 0;  /* Indicate not found */
     }
     /*****************************************************
      * EXTRACT AN ENTRY
@@ -539,19 +587,19 @@ int SQLGetPrivateProfileString( LPCSTR  pszSection,
         else
         {
             iniValue( hIni, szValue );
-	    if ( pRetBuffer ) 
-	    {
-	        strncpy( pRetBuffer, szValue, nRetBuffer );
-	        pRetBuffer[ nRetBuffer - 1 ] = '\0';
-	    }
+	        if ( pRetBuffer ) 
+	        {
+	            strncpy( pRetBuffer, szValue, nRetBuffer );
+	            pRetBuffer[ nRetBuffer - 1 ] = '\0';
+	        }
             nBufPos = strlen( szValue );
         }
+        
+        ret = strlen( pRetBuffer );
     }
 
     iniClose( hIni );
 
-    ret = strlen( pRetBuffer );
-
     save_ini_cache( ret, pszSection, pszEntry, pszDefault, pRetBuffer, nRetBuffer, pszFileName );
 
     return ret;
@@ -609,7 +657,10 @@ int  INSTAPI SQLGetPrivateProfileStringW( LPCWSTR lpszSection,
 	{
 		if ( buf && lpszRetBuffer )
 		{
-			_single_copy_to_wide( lpszRetBuffer, buf, ret + 1 );
+            if ( !lpszSection || !lpszEntry )
+                _multi_string_copy_to_wide( lpszRetBuffer, buf, ret );
+            else
+                _single_copy_to_wide( lpszRetBuffer, buf, ret );
 		}
 	}
 
