@@ -58,7 +58,7 @@ char *_getUIPluginName( char *pszName, char *pszUI )
     }
 
     /* default to qt4 */
-    strcpy( pszName, "libodbcinstQ4" );
+    strcpy( pszName, ODBCINSTPLUGIN );
 
     return pszName;
 }
@@ -153,10 +153,17 @@ BOOL SQLManageDataSources( HWND hWnd )
     {
         /* change the name (SQLManageDataSources to ODBCManageDataSources) to prevent us from calling ourself */
         pSQLManageDataSources = (BOOL (*)(HWND))lt_dlsym( hDLL, "ODBCManageDataSources" );
-        if ( pSQLManageDataSources )
-            return pSQLManageDataSources( ( *(hODBCInstWnd->szUI) ? hODBCInstWnd->hWnd : NULL ) );
+        if ( pSQLManageDataSources ) {
+            BOOL ret;
+            ret = pSQLManageDataSources( ( *(hODBCInstWnd->szUI) ? hODBCInstWnd->hWnd : NULL ) );
+
+            lt_dlclose( hDLL );
+            return ret;
+        }
         else
             inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
+
+        lt_dlclose( hDLL );
     }
     else
     {
@@ -169,10 +176,18 @@ BOOL SQLManageDataSources( HWND hWnd )
             /* change the name (SQLManageDataSources to ODBCManageDataSources) to prevent us from calling ourself   */
             /* its only safe to use hWnd if szUI was specified by the caller                                        */
             pSQLManageDataSources = (BOOL (*)(HWND))lt_dlsym( hDLL, "ODBCManageDataSources" );
-            if ( pSQLManageDataSources )
-                return pSQLManageDataSources( ( *(hODBCInstWnd->szUI) ? hODBCInstWnd->hWnd : NULL ) );
+            if ( pSQLManageDataSources ) {
+                BOOL ret;
+
+                ret = pSQLManageDataSources( ( *(hODBCInstWnd->szUI) ? hODBCInstWnd->hWnd : NULL ) );
+
+                lt_dlclose( hDLL );
+                return ret;
+            }
             else
                 inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
+
+            lt_dlclose( hDLL );
         }
         else
             inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
